==> libphidget22java-1.0.0.20171026.tar.gz: libphidget22java-1.0.0.20171026/src/jni/phidget_jni.c <==
/*
 * This file is part of libphidget22java
 *
 * Copyright 2015 Phidgets Inc <patrick@phidgets.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, see
 * <http://www.gnu.org/licenses/>
 */

#include "phidget_jni.h"

JavaVM *ph_vm = 0;

jclass ph_exception_class;
jmethodID ph_exception_cons;

static jclass asyncResult_class;
static jmethodID asyncResult_cons;
static jclass asyncListener_class;
static jmethodID onAsyncCallback_mid;

//TODO
//#ifdef _WINDOWS
//
//extern int useThreadSecurity;
//extern void destroyThreadSecurityAttributes();
//
//#endif

typedef void(CCONV *mos_task_thread_exit_callback)	(void *ctx);
extern void mos_task_set_thread_exit_handler(mos_task_thread_exit_callback fptr, void *ctx);

static void detachCurrentThreadFromJavaVM(void *ctx)
{
	JNIEnv *env;
	if (ph_vm != NULL && *ph_vm != NULL)
	{
		if ((*ph_vm)->GetEnv(ph_vm, (void **)&env, JNI_VERSION_1_4) != JNI_EDETACHED)
		{
			(*ph_vm)->DetachCurrentThread(ph_vm);
		}
	}
}

jint JNICALL
JNI_OnLoad(JavaVM *vm, void *reserved)
{
	JNIEnv *env;
	jint version = 0;

	ph_vm = vm;

	//TODO
	//// Don't set thread security when we're using Java -  this break events.
	//#ifdef _WINDOWS
	//	useThreadSecurity = PFALSE;
	//	destroyThreadSecurityAttributes();
	//#endif

	if((*ph_vm)->GetEnv(ph_vm, (void **)&env, JNI_VERSION_1_4) != JNI_OK)
		return -1;

	if(!(version = (*env)->GetVersion(env)))
		JNI_ABORT_STDERR("Couldn't get version\nJNI Version: 0x%08x", version);

	//PhidgetException
	if (!(ph_exception_class = (*env)->FindClass(env, "com/phidget22/PhidgetException")))
		JNI_ABORT_STDERR("Coulnd't FindClass com/phidget22/PhidgetException");
	if (!(ph_exception_class = (jclass)(*env)->NewGlobalRef(env, ph_exception_class)))
		JNI_ABORT_STDERR("Couldn't create global ref ph_exception_class");
	if (!(ph_exception_cons = (*env)->GetMethodID(env, ph_exception_class, "<init>", "(ILjava/lang/String;)V")))
		JNI_ABORT_STDERR("Couldn't get Method ID <init> from ph_exception_class");

	if (!(asyncResult_class = (*env)->FindClass(env, "com/phidget22/AsyncResult")))
		JNI_ABORT_STDERR("Couldn't FindClass com/phidget22/AsyncResult");
	if (!(asyncResult_class = (jclass)(*env)->NewGlobalRef(env, asyncResult_class)))
		JNI_ABORT_STDERR("Couldn't create global ref AsyncResult_class");
	if (!(asyncResult_cons = (*env)->GetMethodID(env, asyncResult_class, "<init>", "(Lcom/phidget22/Phidget;Lcom/phidget22/ErrorCode;)V")))
		JNI_ABORT_STDERR("Couldn't get method ID <init> from AsyncResult_class");

	if (!(asyncListener_class = (*env)->FindClass(env, "com/phidget22/AsyncListener")))
		JNI_ABORT_STDERR("Couldn't FindClass com/phidget22/AsyncListener");
	if (!(asyncListener_class = (jclass)(*env)->NewGlobalRef(env, asyncListener_class)))
		JNI_ABORT_STDERR("Couldn't create global ref asyncListener_class");
	if (!(onAsyncCallback_mid = (*env)->GetMethodID(env, asyncListener_class, "onAsyncCallback", "(Lcom/phidget22/AsyncResult;)V")))
		JNI_ABORT_STDERR("Couldn't get method ID onAsyncCallback");

	com_phidget22_OnLoad(env);
	phidget_enums_OnLoad(env);
	phidget_structs_OnLoad(env);

	mos_task_set_thread_exit_handler(detachCurrentThreadFromJavaVM, NULL);

#ifdef _ANDROID
	if(com_phidget22_usb_Phidget_OnLoad(env) && com_phidget22_usb_Manager_OnLoad(env))
		AndroidUsbJarAvailable = PTRUE;
	else
		AndroidUsbJarAvailable = PFALSE;
#endif

	return JNI_VERSION_1_4;
}

jlong
updateGlobalRef(JNIEnv *env, jobject obj, jfieldID fid, jboolean b)
{
	/*
	 * Manages the global reference held by phidget22 to the handler
	 * target.
	 */
	jlong gr;

	if ((gr = (*env)->GetLongField(env, obj, fid)) != 0)
		(*env)->DeleteGlobalRef(env, (jobject)(uintptr_t)gr);
	gr = b ? (jlong)(uintptr_t)(*env)->NewGlobalRef(env, obj) : 0;
	(*env)->SetLongField(env, obj, fid, gr);

	return gr;
}

void asyncCallbackHandler(PhidgetHandle phid, void *arg, PhidgetReturnCode returnCode) {
	JNIEnv *env;
	jobject asyncResult;
	jobject listener;
	jobject phidget;
	jobjectArray arr;

	if ((*ph_vm)->AttachCurrentThreadAsDaemon(ph_vm, (JNIEnvPtr)&env, NULL))
		JNI_ABORT_STDERR("Couldn't AttachCurrentThread");

	arr = (jobjectArray)arg;
	phidget = (*env)->GetObjectArrayElement(env, arr, 0);
	listener = (*env)->GetObjectArrayElement(env, arr, 1);

	if (!(asyncResult = (*env)->NewObject(env, asyncResult_class, asyncResult_cons, phidget, (*env)->CallStaticObjectMethod(env, ErrorCode_class, ErrorCode_getEnum_mid, returnCode))))
		JNI_ABORT_STDERR("Couldn't create new AsyncResult object");

	(*env)->CallVoidMethod(env, listener, onAsyncCallback_mid, asyncResult);
	(*env)->DeleteLocalRef(env, asyncResult);
	(*env)->DeleteGlobalRef(env, phidget);
	(*env)->DeleteGlobalRef(env, listener);
	(*env)->DeleteGlobalRef(env, arr);
}

==> libphidget22java-1.0.0.20180327.tar.gz: libphidget22java-1.0.0.20180327/src/jni/phidget_jni.c <==
/*
 * This file is part of libphidget22java
 *
 * Copyright 2015 Phidgets Inc <patrick@phidgets.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, see
 * <http://www.gnu.org/licenses/>
 */

#include "phidget_jni.h"

JavaVM *ph_vm = 0;

jclass ph_exception_class;
jmethodID ph_exception_cons;

static jclass asyncResult_class;
static jmethodID asyncResult_cons;
static jclass asyncListener_class;
static jmethodID onAsyncCallback_mid;

//TODO
//#ifdef _WINDOWS
//
//extern int useThreadSecurity;
//extern void destroyThreadSecurityAttributes();
//
//#endif

typedef void(CCONV *mos_task_thread_exit_callback)	(void *ctx);
extern void mos_task_set_thread_exit_handler(mos_task_thread_exit_callback fptr, void *ctx);

static void detachCurrentThreadFromJavaVM(void *ctx)
{
	JNIEnv *env;
	if (ph_vm != NULL && *ph_vm != NULL)
	{
		if ((*ph_vm)->GetEnv(ph_vm, (void **)&env, JNI_VERSION_1_4) != JNI_EDETACHED)
		{
			(*ph_vm)->DetachCurrentThread(ph_vm);
		}
	}
}

jint JNICALL
JNI_OnLoad(JavaVM *vm, void *reserved)
{
	JNIEnv *env;
	jint version = 0;

	ph_vm = vm;

	//TODO
	//// Don't set thread security when we're using Java -  this break events.
	//#ifdef _WINDOWS
	//	useThreadSecurity = PFALSE;
	//	destroyThreadSecurityAttributes();
	//#endif

	if((*ph_vm)->GetEnv(ph_vm, (void **)&env, JNI_VERSION_1_4) != JNI_OK)
		return -1;

	if(!(version = (*env)->GetVersion(env)))
		JNI_ABORT_STDERR("Couldn't get version\nJNI Version: 0x%08x", version);

	//PhidgetException
	if (!(ph_exception_class = (*env)->FindClass(env, "com/phidget22/PhidgetException")))
		JNI_ABORT_STDERR("Coulnd't FindClass com/phidget22/PhidgetException");
	if (!(ph_exception_class = (jclass)(*env)->NewGlobalRef(env, ph_exception_class)))
		JNI_ABORT_STDERR("Couldn't create global ref ph_exception_class");
	if (!(ph_exception_cons = (*env)->GetMethodID(env, ph_exception_class, "<init>", "(ILjava/lang/String;)V")))
		JNI_ABORT_STDERR("Couldn't get Method ID <init> from ph_exception_class");

	if (!(asyncResult_class = (*env)->FindClass(env, "com/phidget22/AsyncResult")))
		JNI_ABORT_STDERR("Couldn't FindClass com/phidget22/AsyncResult");
	if (!(asyncResult_class = (jclass)(*env)->NewGlobalRef(env, asyncResult_class)))
		JNI_ABORT_STDERR("Couldn't create global ref AsyncResult_class");
	if (!(asyncResult_cons = (*env)->GetMethodID(env, asyncResult_class, "<init>", "(Lcom/phidget22/Phidget;Lcom/phidget22/ErrorCode;)V")))
		JNI_ABORT_STDERR("Couldn't get method ID <init> from AsyncResult_class");

	if (!(asyncListener_class = (*env)->FindClass(env, "com/phidget22/AsyncListener")))
		JNI_ABORT_STDERR("Couldn't FindClass com/phidget22/AsyncListener");
	if (!(asyncListener_class = (jclass)(*env)->NewGlobalRef(env, asyncListener_class)))
		JNI_ABORT_STDERR("Couldn't create global ref asyncListener_class");
	if (!(onAsyncCallback_mid = (*env)->GetMethodID(env, asyncListener_class, "onAsyncCallback", "(Lcom/phidget22/AsyncResult;)V")))
		JNI_ABORT_STDERR("Couldn't get method ID onAsyncCallback");

	com_phidget22_OnLoad(env);
	phidget_enums_OnLoad(env);
	phidget_structs_OnLoad(env);

	mos_task_set_thread_exit_handler(detachCurrentThreadFromJavaVM, NULL);

#ifdef _ANDROID
	if(com_phidget22_usb_Phidget_OnLoad(env) && com_phidget22_usb_Manager_OnLoad(env))
		AndroidUsbJarAvailable = PTRUE;
	else
		AndroidUsbJarAvailable = PFALSE;
#endif

	return JNI_VERSION_1_4;
}

jlong
updateGlobalRef(JNIEnv *env, jobject obj, jfieldID fid, jboolean b)
{
	/*
	 * Manages the global reference held by phidget22 to the handler
	 * target.
	 */
	jlong gr;

	if ((gr = (*env)->GetLongField(env, obj, fid)) != 0)
		(*env)->DeleteGlobalRef(env, (jobject)(uintptr_t)gr);
	gr = b ? (jlong)(uintptr_t)(*env)->NewGlobalRef(env, obj) : 0;
	(*env)->SetLongField(env, obj, fid, gr);

	return gr;
}

void asyncCallbackHandler(PhidgetHandle phid, void *arg, PhidgetReturnCode returnCode) {
	JNIEnv *env;
	jobject asyncResult;
	jobject listener;
	jobject phidget;
	jobjectArray arr;
	jobject errorCodeEnum;

	if ((*ph_vm)->AttachCurrentThreadAsDaemon(ph_vm, (JNIEnvPtr)&env, NULL))
		JNI_ABORT_STDERR("Couldn't AttachCurrentThread");

	arr = (jobjectArray)arg;
	phidget = (*env)->GetObjectArrayElement(env, arr, 0);
	listener = (*env)->GetObjectArrayElement(env, arr, 1);
	errorCodeEnum = (*env)->CallStaticObjectMethod(env, ErrorCode_class, ErrorCode_getEnum_mid, returnCode);

	if (!(asyncResult = (*env)->NewObject(env, asyncResult_class, asyncResult_cons, phidget, errorCodeEnum)))
		JNI_ABORT_STDERR("Couldn't create new AsyncResult object");

	(*env)->CallVoidMethod(env, listener, onAsyncCallback_mid, asyncResult);

	(*env)->DeleteGlobalRef(env, errorCodeEnum);
	(*env)->DeleteLocalRef(env, asyncResult);
	(*env)->DeleteGlobalRef(env, phidget);
	(*env)->DeleteGlobalRef(env, listener);
	(*env)->DeleteGlobalRef(env, arr);
}
